@@ -11,6 +11,7 @@ STATE_UNKNOWN = "unknown"
 
 # Supported climate features
 SUPPORT_TARGET_TEMPERATURE = 1
+SUPPORT_FAN_MODE = 8
 SUPPORT_PRESET_MODE = 16
 
 # Supported cover features
@@ -21,14 +22,27 @@ SUPPORT_SET_POSITION = 4
 # HVAC modes
 HVAC_MODE_OFF = "off"
 HVAC_MODE_HEAT = "heat"
+HVAC_MODE_COOL = "cool"
 HVAC_MODE_AUTO = "auto"
 
 # HVAC states
 CURRENT_HVAC_OFF = "off"
 CURRENT_HVAC_HEAT = "heating"
+CURRENT_HVAC_HEAT_IDLE = "heating (idling)"
+CURRENT_HVAC_COOL = "cooling"
+CURRENT_HVAC_COOL_IDLE = "cooling (idling)"
 CURRENT_HVAC_IDLE = "idle"
 
 # Supported presets
 PRESET_FOLLOW_SCHEDULE = "Follow Schedule"
 PRESET_PERMANENT_HOLD = "Permanent Hold"
+PRESET_TEMPORARY_HOLD = "Temporary Hold"
+PRESET_ECO = "Eco"
 PRESET_OFF = "Off"
+
+# Supported fan modes
+FAN_MODE_AUTO = "Auto"
+FAN_MODE_HIGH = "High"
+FAN_MODE_MEDIUM = "Medium"
+FAN_MODE_LOW = "Low"
+FAN_MODE_OFF = "Off"
